﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.IO;

namespace OledBm {
    public partial class FrmMain : Form {
        Bitmap mBitmapOriginal;
        Bitmap mBitmapOled;
        byte[] mBinaerCode;

        public FrmMain() {
            InitializeComponent();
        }

        /// <summary>
        /// Prüft die Höhe des Bildes und liefert true, wenn die Höhe durch 8 teilbar ist.
        /// </summary>
        /// <returns></returns>
        bool BildPruefen() {
            if (mBitmapOriginal.Height % 8 == 0)
                return true;
            else {
                MessageBox.Show("Die Bildhöhe muss glatt durch 8 teilbar sein!");
                return false;
            }
        }

        private void BtnDateiWaehlen_Click(object sender, EventArgs e) {
            if (ofdBitmap.ShowDialog() == DialogResult.OK) {
                try {
                    mBitmapOriginal = new Bitmap(ofdBitmap.FileName);
                }
                catch {
                    MessageBox.Show("Bild konnte nicht geladen werden!");
                    return;
                }

                pbxBitmapDatei.Image = mBitmapOriginal;
                lblGroesse.Text = "Bildgröße: " + mBitmapOriginal.Width + " x " + mBitmapOriginal.Height;

                BildPruefen();

                gpbUmwandlung.Enabled = true;
                BtnErstelleCode.Enabled = true;
                BtnCopyClpbrd.Enabled = true;

                TrbR_ValueChanged(null, null);
            }
        }


        /// <summary>
        /// Erstellt den Code für den AVR für das übergebene Bitmap als Programmcode und Binärcode.
        /// </summary>
        /// <param name="bitmap"></param>
        /// <param name="progCode"></param>
        /// <param name="binCode"></param>
        void ProgMemCode(Bitmap bitmap, out string progCode, out byte[] binCode) {
            int x, y, Reihe;
            byte bytewert;
            string zeile;
            int n;

            // Rückgabewerte, wenn Bild das falsche Format hat
            progCode = "";
            binCode = null;

            // Bei falschem Bildformat Ende
            if (!BildPruefen())
                return;

            // Array für Binärcode mit Größe Width * Height / 8 
            binCode = new byte[bitmap.Width * bitmap.Height / 8];

            // Code für C und Binärdatei generieren
            progCode = "";
            n = 0;
            for (Reihe = 0; Reihe < bitmap.Height / 8; Reihe++) {
                if (Reihe > 0)
                    progCode += Environment.NewLine;

                zeile = "";
                for (x = 0; x < bitmap.Width; x++) {
                    bytewert = 0;
                    for (y = (Reihe + 1) * 8 - 1; y >= Reihe * 8; y--) {
                        bytewert <<= 1;
                        if ((bitmap.GetPixel(x, y).R != 0) || (bitmap.GetPixel(x, y).G != 0) || (bitmap.GetPixel(x, y).B != 0))
                            bytewert += 1;
                    }
                    zeile += "0x" + bytewert.ToString("X2") + ", ";
                    binCode[n++] = bytewert;
                }
                progCode += zeile;
            }
            progCode = progCode.Remove(progCode.Length - 2);

            // Zusatz für C-Code einfügen
            progCode = "static const uint8_t bitmap[] PROGMEM = {" + Environment.NewLine + 
                     "0x" + bitmap.Width.ToString("X2") + ", 0x" + bitmap.Height.ToString("X2") + ", " +
                     progCode + Environment.NewLine +
                     "};";
        }

        /// <summary>
        /// Monochrom-Bitmap für Oled aus übergebener Bitmap erstellen.
        /// Die Bitmap wird dabei verändert.
        /// </summary>
        /// <param name="bitmap"></param>
        /// <param name="R"></param>
        /// <param name="G"></param>
        /// <param name="B"></param>
        /// <param name="invertiert"></param>
        void Schwelle(Bitmap bitmap, int R, int G, int B, bool invertiert) {
            Color col;

            for (int x = 0; x < bitmap.Width; x++)
                for(int y = 0; y < bitmap.Height; y++) {
                    col = bitmap.GetPixel(x, y);

                    if (col.R > R && col.G > G && col.B > B)
                        bitmap.SetPixel(x, y, invertiert ? Color.White : Color.Black);
                    else
                        bitmap.SetPixel(x, y, invertiert ? Color.Black : Color.White);
                }
        }

        private void TrbR_ValueChanged(object sender, EventArgs e) {
            Cursor.Current = Cursors.WaitCursor;

            mBitmapOled = new Bitmap(mBitmapOriginal);
            Schwelle(mBitmapOled, trbR.Value, trbG.Value, trbB.Value, ckbInvertiert.Checked);
            pbxBitmapOled.Image = mBitmapOled;

            Cursor.Current = Cursors.Default;
        }

        private void BtnErstelleCode_Click(object sender, EventArgs e) {
            ProgMemCode(mBitmapOled, out string progCode, out mBinaerCode);
            tbxProgCode.Text = progCode;

            if (progCode != "")
                BtnBinaerdateiSpeichern.Enabled = true;
        }

        private void BtnCopyClpbrd_Click(object sender, EventArgs e) {
            Clipboard.SetText(tbxProgCode.Text);
        }

        private void BtnEnde_Click(object sender, EventArgs e) {
            Close();
        }

        private void BtnBinaerdateiSpeichern_Click(object sender, EventArgs e) {
            if (sfdBinaerdatei.ShowDialog() == DialogResult.OK) 
                File.WriteAllBytes(sfdBinaerdatei.FileName, mBinaerCode);
        }

        private void OfdBitmap_FileOk(object sender, CancelEventArgs e) {

        }
    }
}
